<?php

namespace Drupal\Tests\protected_forms\Functional;

use Drupal\comment\Tests\CommentTestTrait;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\node\Traits\NodeCreationTrait;
use Drupal\user\RoleInterface;
use Symfony\Component\HttpFoundation\Response;

/**
 * {@inheritdoc}
 */
class ProtectedFormsTest extends BrowserTestBase {
  use NodeCreationTrait, CommentTestTrait;
  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';
  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'node',
    'comment',
    'protected_forms',
    'dblog',
  ];

  /**
   * A user with authenticated permissions.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $user;

  /**
   * {@inheritDoc}
   */
  protected function setUp(): void {
    parent::setUp();
    // Create an article content type.
    $this->createContentType([
      'type' => 'article',
      'name' => 'Article',
    ]);

    // Create comment field on article.
    $this->addDefaultCommentField('node', 'article');

    // Create an article node.
    $this->drupalCreateNode([
      'title' => 'An Article',
      'type' => 'article',
    ])->save();

    // Allow authenticated users to post comments and change the settings.
    $permissions = [
      'access content',
      'access comments',
      'post comments',
      'skip comment approval',
      'administer protected forms',
      'access site reports',
    ];
    user_role_grant_permissions(RoleInterface::AUTHENTICATED_ID, $permissions);
    $this->user = $this->drupalCreateUser([]);
    $this->drupalLogin($this->user);
  }

  /**
   * {@inheritdoc}
   */
  public function testNodeCommentRejectedSubmissionPrintsCorrectMessage() {
    // Add a custom string to the reject patterns list.
    $rejectPatterns = $this->config('protected_forms.settings')->get('protected_forms.reject_patterns');
    $inputString = ' monkey';
    $rejectPatterns[] = $inputString;
    $this->config('protected_forms.settings')
      ->set('protected_forms.reject_patterns', $rejectPatterns)
      ->save();

    // Attempt to post a spam comment using the newly added reject pattern.
    $input = [
      'edit-comment-body-0-value' => $inputString,
    ];
    $this->drupalGet('node/1');
    $this->submitForm($input, 'Save');

    // Ensure we get a 200 response.
    $this->assertSession()->statusCodeEquals(Response::HTTP_OK);

    // Ensure the rejected message is displayed.
    $message = $this->config('protected_forms.settings')->get('protected_forms.reject_message');
    $this->assertSession()->responseContains($message);
    $this->assertSession()->responseNotContains('Your comment has been posted.');

    // Attempt to post a normal comment.
    $input = [
      'edit-comment-body-0-value' => 'a normal comment',
    ];
    $this->drupalGet('node/1');
    $this->submitForm($input, 'Save');

    // Ensure we get a 200 response.
    $this->assertSession()->statusCodeEquals(Response::HTTP_OK);

    // Ensure our normal post was posted.
    $this->assertSession()->responseContains('Your comment has been posted.');
    $this->assertSession()->responseContains('a normal comment');

    // Attempt to post a comment that contains a phrase blocked by default.
    $input = [
      'edit-comment-body-0-value' => 'You can win something here!',
    ];
    $this->drupalGet('node/1');
    $this->submitForm($input, 'Save');

    // Ensure we get a 200 response.
    $this->assertSession()->statusCodeEquals(Response::HTTP_OK);

    // Ensure the rejected message is displayed.
    $message = $this->config('protected_forms.settings')->get('protected_forms.reject_message');
    $this->assertSession()->responseContains($message);
    $this->assertSession()->responseNotContains('Your comment has been posted.');

    // Attempt to post a comment that contains multiple blocked phrases.
    $input = [
      'edit-comment-body-0-value' => 'You can win this for free or buy it!',
    ];
    $this->drupalGet('node/1');
    $this->submitForm($input, 'Save');

    // Ensure we get a 200 response.
    $this->assertSession()->statusCodeEquals(Response::HTTP_OK);

    // Ensure the rejected message is displayed.
    $message = $this->config('protected_forms.settings')->get('protected_forms.reject_message');
    $this->assertSession()->responseContains($message);
    $this->assertSession()->responseNotContains('Your comment has been posted.');

    // Attempt to post a comment with only a partial match that should be valid
    // because only whole patterns are matched.
    // (Whole pattern: "are whitelisted")
    $input = [
      'edit-comment-body-0-value' => 'I am whitelisted.',
    ];
    $this->drupalGet('node/1');
    $this->submitForm($input, 'Save');

    // Ensure we get a 200 response.
    $this->assertSession()->statusCodeEquals(Response::HTTP_OK);

    // Ensure our normal post was posted.
    $this->assertSession()->responseContains('Your comment has been posted.');
    $this->assertSession()->responseContains('I am whitelisted.');
  }

  /**
   * Tests that excluded forms are not protected.
   */
  public function testExcludedFormsNotProtected() {
    $this->drupalLogout();
    // Enter spam on the user registration form, which is excluded by default.
    $input = [
      'mail' => 'admin@admin123.com',
      'name' => 'free win',
    ];
    $this->drupalGet('/user/register');
    $this->submitForm($input, 'Create new account');
    $this->assertSession()->responseContains('A welcome message with further instructions has been sent to your email address.');
    $this->assertSession()->responseNotContains('Oops! It looks like your post contains spam content.');
    // Remove the account creation from from the excluded forms and try again.
    \Drupal::configFactory()->getEditable('protected_forms.settings')
      ->set('protected_forms.excluded_forms', [])
      ->save();
    $this->drupalGet('/user/register');
    $this->submitForm($input, 'Create new account');
    $this->assertSession()->responseNotContains('A welcome message with further instructions has been sent to your email address.');
    $this->assertSession()->responseContains('Oops! It looks like your post contains spam content.');
  }

  /**
   * Tests the functionality of the language script validation.
   */
  public function testLanguageScriptValidation() {
    // Disallow the use of any characters except latin in config.
    $this->config('protected_forms.settings')
      ->set('protected_forms.allowed_scripts', ['Latin'])
      ->save();

    // Attempt to post a comment containing greek characters.
    $inputString = 'ΓΩ';
    $input = [
      'edit-comment-body-0-value' => $inputString,
    ];
    $this->drupalGet('node/1');
    $this->submitForm($input, 'Save');

    // Ensure we get a 200 response.
    $session = $this->assertSession();
    $session->statusCodeEquals(Response::HTTP_OK);

    // Ensure the rejected message is displayed.
    $message = $this->config('protected_forms.settings')->get('protected_forms.reject_message');
    $session->responseContains($message);
    $session->responseNotContains('Your comment has been posted.');

    // Allow greek characters in config.
    $this->config('protected_forms.settings')
      ->set('protected_forms.allowed_scripts', ['Greek'])
      ->save();

    // Attempt to post a comment containing greek characters again.
    $input = [
      'edit-comment-body-0-value' => $inputString,
    ];
    $this->drupalGet('node/1');
    $this->submitForm($input, 'Save');

    // Ensure we get a 200 response.
    $session->statusCodeEquals(Response::HTTP_OK);

    // Ensure our normal post was posted.
    $session->responseContains('Your comment has been posted.');
    $session->responseContains($inputString);
    $this->drupalGet('node/1');
    $session->statusCodeEquals(Response::HTTP_OK);
    $session->pageTextContains($inputString);
  }

  /**
   * Tests that the reject message can be customized.
   */
  public function testRejectMessage() {
    // Change the default reject message in config.
    $this->config('protected_forms.settings')
      ->set('protected_forms.reject_message', 'This comment is rejected because of spam contents.')
      ->save();

    // Attempt to post a spam comment.
    $inputString = 'free win buy';
    $input = [
      'edit-comment-body-0-value' => $inputString,
    ];
    $this->drupalGet('node/1');
    $this->submitForm($input, 'Save');

    // Ensure we get a 200 response.
    $session = $this->assertSession();
    $session->statusCodeEquals(Response::HTTP_OK);

    // Ensure the updated rejected message is displayed.
    $session->responseContains('This comment is rejected because of spam contents.');
    $session->responseNotContains('Your comment has been posted.');

  }

  /**
   * Tests that log messages are being added correctly.
   */
  public function testLogMessages() {
    // Attempt to post a few spam comments.
    $input = [
      'edit-comment-body-0-value' => 'free win buy',
    ];
    $this->drupalGet('node/1');
    $this->submitForm($input, 'Save');
    $input = [
      'edit-comment-body-0-value' => 'ΓΩ',
    ];
    $this->drupalGet('node/1');
    $this->submitForm($input, 'Save');

    // Go to the log page and check that the correct messages are present.
    $this->drupalGet('admin/reports/dblog');
    $session = $this->assertSession();
    $session->statusCodeEquals(Response::HTTP_OK);
    $session->responseContains('Rejected language:  ΓΩ');
    $session->responseContains('Rejected pattern:   free win buy');
  }

}
